@@ -1,6 +1,3 @@
-/* $XdotOrg: xc/lib/font/fontfile/fontfile.c,v 1.4 2005/07/03 07:01:00 daniels Exp $ */
-/* $Xorg: fontfile.c,v 1.4 2001/02/09 02:04:03 xorgcvs Exp $ */
-
 /*
 
 Copyright 1991, 1998  The Open Group
@@ -26,36 +23,56 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/lib/font/fontfile/fontfile.c,v 3.21 2003/12/02 19:50:40 dawes Exp $ */
 
 /*
  * Author:  Keith Packard, MIT X Consortium
  */
-/* $NCDXorg: @(#)fontfile.c,v 1.6 1991/07/02 17:00:46 lemke Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#include    <X11/fonts/fntfilst.h>
+#include <X11/fonts/fntfilst.h>
+#include <X11/keysym.h>
 #ifdef WIN32
 #include <ctype.h>
 #endif
 
+static unsigned char
+ISOLatin1ToLower(unsigned char source)
+{
+    if (source >= XK_A && source <= XK_Z)
+	return source + (XK_a - XK_A);
+    if (source >= XK_Agrave && source <= XK_Odiaeresis)
+	return source + (XK_agrave - XK_Agrave);
+    if (source >= XK_Ooblique && source <= XK_Thorn)
+	return source + (XK_oslash - XK_Ooblique);
+    return source;
+}
+
+_X_HIDDEN void
+CopyISOLatin1Lowered(char *dest, const char *source, int length)
+{
+    int i;
+    for (i = 0; i < length; i++, source++, dest++)
+	*dest = ISOLatin1ToLower(*source);
+    *dest = '\0';
+}
+
 /*
  * Map FPE functions to renderer functions
  */
 
-static int FontFileOpenBitmapNCF (FontPathElementPtr fpe, FontPtr *pFont, 
-				  int flags, FontEntryPtr entry, 
-				  fsBitmapFormat format, 
+static int FontFileOpenBitmapNCF (FontPathElementPtr fpe, FontPtr *pFont,
+				  int flags, FontEntryPtr entry,
+				  fsBitmapFormat format,
 				  fsBitmapFormatMask fmask,
 				  FontPtr non_cachable_font);
 
 int
-FontFileNameCheck (char *name)
+FontFileNameCheck (const char *name)
 {
 #ifndef NCD
-#if defined(__UNIXOS2__) || defined(WIN32)
+#if defined(WIN32)
     /* OS/2 uses D:/... as a path name for fonts, so accept this as a valid
      * path if it starts with a letter and a colon. Same applies for WIN32
      */
@@ -102,14 +119,14 @@ FontFileResetFPE (FontPathElementPtr fpe)
     if (FontFileDirectoryChanged (dir))
     {
 	/* can't do it, so tell the caller to close and re-open */
-	return FPEResetFailed;	
+	return FPEResetFailed;
     }
-    else 
+    else
     {
 	if (dir->nonScalable.used > 0)
 	    if (!FontFileRegisterBitmapSource (fpe))
 	    {
-	        return FPEResetFailed;	
+	        return FPEResetFailed;
 	    }
         return Successful;
     }
@@ -124,7 +141,7 @@ FontFileFreeFPE (FontPathElementPtr fpe)
 }
 
 static int
-transfer_values_to_alias(char *entryname, int entrynamelength, 
+transfer_values_to_alias(char *entryname, int entrynamelength,
 			 char *resolvedname,
 			 char **aliasName, FontScalablePtr vals)
 {
@@ -236,10 +253,10 @@ transfer_values_to_alias(char *entryname, int entrynamelength,
 
 /* ARGSUSED */
 int
-FontFileOpenFont (pointer client, FontPathElementPtr fpe, Mask flags, 
-		  char *name, int namelen, 
+FontFileOpenFont (pointer client, FontPathElementPtr fpe, Mask flags,
+		  const char *name, int namelen,
 		  fsBitmapFormat format, fsBitmapFormatMask fmask,
-		  XID id, FontPtr *pFont, char **aliasName, 
+		  XID id, FontPtr *pFont, char **aliasName,
 		  FontPtr non_cachable_font)
 {
     FontDirectoryPtr	dir;
@@ -255,7 +272,7 @@ FontFileOpenFont (pointer client, FontPathElementPtr fpe, Mask flags,
     Bool		noSpecificSize;
     int			nranges;
     fsRange		*ranges;
-    
+
     if (namelen >= MAXFONTNAMELEN)
 	return AllocError;
     dir = (FontDirectoryPtr) fpe->private;
@@ -303,17 +320,6 @@ FontFileOpenFont (pointer client, FontPathElementPtr fpe, Mask flags,
 				     entry->u.alias.resolved, aliasName, &vals);
 	    ret = FontNameAlias;
 	    break;
-#ifdef NOTYET
-	case FONT_ENTRY_BC:
-	    bc = &entry->u.bc;
-	    entry = bc->entry;
-	    ret = (*scalable->renderer->OpenScalable)
-		    (fpe, pFont, flags, entry, &bc->vals, format, fmask,
-		     non_cachable_font);
-	    if (ret == Successful && *pFont)
-		(*pFont)->fpe = fpe;
-	    break;
-#endif
 	default:
 	    ret = BadFontName;
 	}
@@ -325,7 +331,7 @@ FontFileOpenFont (pointer client, FontPathElementPtr fpe, Mask flags,
 
     if (ret != BadFontName)
     {
-	if (ranges) xfree(ranges);
+	if (ranges) free(ranges);
 	return ret;
     }
 
@@ -476,7 +482,7 @@ FontFileOpenFont (pointer client, FontPathElementPtr fpe, Mask flags,
 	ret = BadFontName;
 
     if (ranges)
-	xfree(ranges);
+	free(ranges);
     return ret;
 }
 
@@ -504,8 +510,8 @@ FontFileCloseFont (FontPathElementPtr fpe, FontPtr pFont)
 }
 
 static int
-FontFileOpenBitmapNCF (FontPathElementPtr fpe, FontPtr *pFont, 
-		       int flags, FontEntryPtr entry, 
+FontFileOpenBitmapNCF (FontPathElementPtr fpe, FontPtr *pFont,
+		       int flags, FontEntryPtr entry,
 		       fsBitmapFormat format, fsBitmapFormatMask fmask,
 		       FontPtr non_cachable_font)
 {
@@ -522,7 +528,7 @@ FontFileOpenBitmapNCF (FontPathElementPtr fpe, FontPtr *pFont,
 	return BadFontName;
     strcpy (fileName, dir->directory);
     strcat (fileName, bitmap->fileName);
-    ret = (*bitmap->renderer->OpenBitmap) 
+    ret = (*bitmap->renderer->OpenBitmap)
 			(fpe, pFont, flags, entry, fileName, format, fmask,
 			 non_cachable_font);
     if (ret == Successful)
@@ -534,8 +540,8 @@ FontFileOpenBitmapNCF (FontPathElementPtr fpe, FontPtr *pFont,
 }
 
 int
-FontFileOpenBitmap (FontPathElementPtr fpe, FontPtr *pFont, 
-		    int flags, FontEntryPtr entry, 
+FontFileOpenBitmap (FontPathElementPtr fpe, FontPtr *pFont,
+		    int flags, FontEntryPtr entry,
 		    fsBitmapFormat format, fsBitmapFormatMask fmask)
 {
     return FontFileOpenBitmapNCF (fpe, pFont, flags, entry, format, fmask,
@@ -543,7 +549,7 @@ FontFileOpenBitmap (FontPathElementPtr fpe, FontPtr *pFont,
 }
 
 static int
-FontFileGetInfoBitmap (FontPathElementPtr fpe, FontInfoPtr pFontInfo, 
+FontFileGetInfoBitmap (FontPathElementPtr fpe, FontInfoPtr pFontInfo,
 		       FontEntryPtr entry)
 {
     FontBitmapEntryPtr	bitmap;
@@ -564,8 +570,8 @@ FontFileGetInfoBitmap (FontPathElementPtr fpe, FontInfoPtr pFontInfo,
 }
 
 static void
-_FontFileAddScalableNames(FontNamesPtr names, FontNamesPtr scaleNames, 
-			  FontNamePtr nameptr, char *zeroChars, 
+_FontFileAddScalableNames(FontNamesPtr names, FontNamesPtr scaleNames,
+			  FontNamePtr nameptr, char *zeroChars,
 			  FontScalablePtr vals, fsRange *ranges,
 			  int nranges, int *max)
 {
@@ -681,8 +687,8 @@ _FontFileAddScalableNames(FontNamesPtr names, FontNamesPtr scaleNames,
 
 /* ARGSUSED */
 static int
-_FontFileListFonts (pointer client, FontPathElementPtr fpe, 
-		    char *pat, int len, int max, FontNamesPtr names, 
+_FontFileListFonts (pointer client, FontPathElementPtr fpe,
+		    const char *pat, int len, int max, FontNamesPtr names,
 		    int mark_aliases)
 {
     FontDirectoryPtr	dir;
@@ -730,7 +736,7 @@ _FontFileListFonts (pointer client, FontPathElementPtr fpe,
 	scaleNames = MakeFontNamesRecord (0);
 	if (!scaleNames)
 	{
-	    if (ranges) xfree(ranges);
+	    if (ranges) free(ranges);
 	    return AllocError;
 	}
 	FontFileFindNamesInScalableDir (&dir->scalable, &zeroName, max,
@@ -747,7 +753,7 @@ _FontFileListFonts (pointer client, FontPathElementPtr fpe,
 	scaleNames = MakeFontNamesRecord (0);
 	if (!scaleNames)
 	{
-	    if (ranges) xfree(ranges);
+	    if (ranges) free(ranges);
 	    return AllocError;
 	}
 	FontFileFindNamesInScalableDir (&dir->nonScalable, &zeroName,
@@ -760,7 +766,7 @@ _FontFileListFonts (pointer client, FontPathElementPtr fpe,
 				  &max);
 	FreeFontNames (scaleNames);
 
-	if (ranges) xfree(ranges);
+	if (ranges) free(ranges);
     }
     else
     {
@@ -788,27 +794,27 @@ typedef struct _LFWIData {
 } LFWIDataRec, *LFWIDataPtr;
 
 int
-FontFileListFonts (pointer client, FontPathElementPtr fpe, char *pat, 
+FontFileListFonts (pointer client, FontPathElementPtr fpe, const char *pat,
 		   int len, int max, FontNamesPtr names)
 {
     return _FontFileListFonts (client, fpe, pat, len, max, names, 0);
 }
 
 int
-FontFileStartListFonts(pointer client, FontPathElementPtr fpe, 
-		       char *pat, int len, int max, 
+FontFileStartListFonts(pointer client, FontPathElementPtr fpe,
+		       const char *pat, int len, int max,
 		       pointer *privatep, int mark_aliases)
 {
     LFWIDataPtr	data;
     int		ret;
 
-    data = (LFWIDataPtr) xalloc (sizeof *data);
+    data = malloc (sizeof *data);
     if (!data)
 	return AllocError;
     data->names = MakeFontNamesRecord (0);
     if (!data->names)
     {
-	xfree (data);
+	free (data);
 	return AllocError;
     }
     ret = _FontFileListFonts (client, fpe, pat, len,
@@ -816,7 +822,7 @@ FontFileStartListFonts(pointer client, FontPathElementPtr fpe,
     if (ret != Successful)
     {
 	FreeFontNames (data->names);
-	xfree (data);
+	free (data);
 	return ret;
     }
     data->current = 0;
@@ -826,8 +832,8 @@ FontFileStartListFonts(pointer client, FontPathElementPtr fpe,
 
 
 int
-FontFileStartListFontsWithInfo(pointer client, FontPathElementPtr fpe, 
-			       char *pat, int len, int max, 
+FontFileStartListFontsWithInfo(pointer client, FontPathElementPtr fpe,
+			       const char *pat, int len, int max,
 			       pointer *privatep)
 {
     return FontFileStartListFonts(client, fpe, pat, len, max, privatep, 0);
@@ -835,8 +841,8 @@ FontFileStartListFontsWithInfo(pointer client, FontPathElementPtr fpe,
 
 /* ARGSUSED */
 static int
-FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe, 
-			     char **namep, int *namelenp, 
+FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
+			     char **namep, int *namelenp,
 			     FontInfoPtr *pFontInfo)
 {
     FontDirectoryPtr	dir;
@@ -852,14 +858,14 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
     Bool		noSpecificSize;
     int			nranges;
     fsRange		*ranges;
-    
+
     char		*name = *namep;
     int			namelen = *namelenp;
-    
+
     if (namelen >= MAXFONTNAMELEN)
 	return AllocError;
     dir = (FontDirectoryPtr) fpe->private;
-    
+
     /* Match non-scalable pattern */
     CopyISOLatin1Lowered (lowerName, name, namelen);
     lowerName[namelen] = '\0';
@@ -876,7 +882,7 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
         tmpName.length = strlen(lowerName);
 	entry = FontFileFindNameInDir (&dir->nonScalable, &tmpName);
     }
-    
+
     if (entry)
     {
 	switch (entry->type) {
@@ -900,23 +906,6 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
 	    *namelenp = strlen (*namep);
 	    ret = FontNameAlias;
 	    break;
-#ifdef NOTYET
-	case FONT_ENTRY_BC:
-	    /* no LFWI for this yet */
-	    bc = &entry->u.bc;
-	    entry = bc->entry;
-	    /* Make a new scaled instance */
-	    if (strlen(dir->directory) + strlen(scalable->fileName) >=
-		sizeof(fileName)) {
-		ret = BadFontName;
-	    } else {
-		strcpy (fileName, dir->directory);
-		strcat (fileName, scalable->fileName);
-		ret = (*scalable->renderer->GetInfoScalable)
-		    (fpe, *pFontInfo, entry, tmpName, fileName, &bc->vals);
-	    }
-	    break;
-#endif
 	default:
 	    ret = BadFontName;
 	}
@@ -925,10 +914,10 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
     {
       ret = BadFontName;
     }
-    
+
     if (ret != BadFontName)
     {
-	if (ranges) xfree(ranges);
+	if (ranges) free(ranges);
 	return ret;
     }
 
@@ -956,7 +945,7 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
 	    tmpName.ndashes = entry->name.ndashes;
 	}
     }
-    
+
     if (entry)
     {
 	noSpecificSize = FALSE;	/* TRUE breaks XLFD enhancements */
@@ -1004,11 +993,6 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
 	    }
 	    else
 	    {
-#ifdef NOTDEF
-		/* no special case yet */
-		ret = FontFileMatchBitmapSource (fpe, pFont, flags, entry, &vals, format, fmask, noSpecificSize);
-		if (ret != Successful)
-#endif
 		{
 		    char origName[MAXFONTNAMELEN];
 
@@ -1017,7 +1001,7 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
 		    vals.xlfdName = origName;
 		    vals.ranges = ranges;
 		    vals.nranges = nranges;
-		    
+
 		    /* Make a new scaled instance */
 		    if (strlen(dir->directory) + strlen(scalable->fileName) >=
 			sizeof(fileName)) {
@@ -1034,7 +1018,7 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
                                 (fpe, *pFontInfo, entry, fileName);
 		    }
 		    if (ranges) {
-			xfree(ranges);
+			free(ranges);
 			ranges = NULL;
 		    }
 		}
@@ -1048,13 +1032,13 @@ FontFileListOneFontWithInfo (pointer client, FontPathElementPtr fpe,
 	ret = BadFontName;
 
     if (ranges)
-	xfree(ranges);
+	free(ranges);
     return ret;
 }
 
 int
-FontFileListNextFontWithInfo(pointer client, FontPathElementPtr fpe, 
-			     char **namep, int *namelenp, 
+FontFileListNextFontWithInfo(pointer client, FontPathElementPtr fpe,
+			     char **namep, int *namelenp,
 			     FontInfoPtr *pFontInfo,
 			     int *numFonts, pointer private)
 {
@@ -1066,7 +1050,7 @@ FontFileListNextFontWithInfo(pointer client, FontPathElementPtr fpe,
     if (data->current == data->names->nnames)
     {
 	FreeFontNames (data->names);
-	xfree (data);
+	free (data);
 	return BadFontName;
     }
     name = data->names->names[data->current];
@@ -1082,15 +1066,15 @@ FontFileListNextFontWithInfo(pointer client, FontPathElementPtr fpe,
 }
 
 int
-FontFileStartListFontsAndAliases(pointer client, FontPathElementPtr fpe, 
-				 char *pat, int len, int max, 
+FontFileStartListFontsAndAliases(pointer client, FontPathElementPtr fpe,
+				 const char *pat, int len, int max,
 				 pointer *privatep)
 {
     return FontFileStartListFonts(client, fpe, pat, len, max, privatep, 1);
 }
 
 int
-FontFileListNextFontOrAlias(pointer client, FontPathElementPtr fpe, 
+FontFileListNextFontOrAlias(pointer client, FontPathElementPtr fpe,
 			    char **namep, int *namelenp, char **resolvedp,
 			    int *resolvedlenp, pointer private)
 {
@@ -1102,7 +1086,7 @@ FontFileListNextFontOrAlias(pointer client, FontPathElementPtr fpe,
     if (data->current == data->names->nnames)
     {
 	FreeFontNames (data->names);
-	xfree (data);
+	free (data);
 	return BadFontName;
     }
     name = data->names->names[data->current];
